/* ******************************************************************** */
/*					VXI-5536 Demo Test									*/
/*                                                                      */
/*  Filename:   5536demo.c												*/
/*																		*/
/*		Demo test routines												*/
/*                                                                      */
/*                                                                      */
/*   Created:   Apr. 29, 1997  CB from 5536test program                 */
/*   Revised:   May  12, 1997  CB added buffer display					*/
/* ******************************************************************** */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <io.h>

#include "..\vxi_lib\vxi_drv.h"			// ICS VXI Library function prototypes

#include "..\vxi_lib\vxicmds.h"			// VXI Std Word Serial Commands
#include "..\vxi_lib\fdc_cmds.h"		// FDC Word Serial Commands


int ModuleInit(void);
int DataTest(void);
void A32_WriteBufr(int LogicalAddr, int chan, int *buffer);
int setupChan(int *TxChan, int *RxChan, long *TxRate);
void XFDC_OpenWriteBufrPS(int LogicalAddr, int chan);
int XFDC_GetHeader(int LogicalAddr, int chan);
void XFDC_ChannelClose(int LogicalAddr, int chan);
void XFDC_OpenReadBufrPS(int LogicalAddr, int chan);
void XFDC_PassBuffer(int LogicalAddr, int chan);
int XFDC_GetChanAddr(int LogicalAddr, int chan, unsigned long *MemAddr);
int A32bufr(long MemAddr);
void pause(void);

int Tx1Buffer[] = {44, 0, 4, 0, 0, 0,
					0x8005, 0x8003, 0x800b, 0x8009,
					0x8005, 0x8003, 0x800b, 0x8009,
					0x8005, 0x8003, 0x800b, 0x8009,
					0x8005, 0x8003, 0x800b, 0x8009};

int Tx2Buffer[] = {44, 0, 4, 0, 0, 0,
					0x8015, 0x8013, 0x801b, 0x8019,
					0x8015, 0x8013, 0x801b, 0x8019,
					0x8015, 0x8013, 0x801b, 0x8019,
					0x8015, 0x8013, 0x801b, 0x8019};

int LogicalAddr;

//extern long WrBusErr, RdBusErr, VErrCnt;

char title[]="VXI-5536 Demo Test Program, Version 1.0, 04/29/97";
char copywrt[]="Copyright (c) 1997, ICS Electronics Corporation";


int debug = 0;
int run;

/* ***************************** main ********************************* */
/* ******************************************************************** */

main()
{
	puts(title);
	puts(copywrt);
	puts("\n\n");

	vxiOpen();				// open ICS VXI-5543 VXI library

	if(!ModuleInit())		// initialize 5536 module
		DataTest();				// if OK run data test

	vxiClose();				// close VXI library

  	exit(0);
}

/* ************************** ModuleInit ****************************** */
/* ******************************************************************** */

int ModuleInit(void)
{
	unsigned RtnValue;
	int Err_Code;
	long slot=0;

	puts("\n Searching commander table...");

	LogicalAddr = vxiGetAddr(536, 1);	// find 1st 5536
	if(LogicalAddr != -1)
	{
		Err_Code = vxiGetAttrib(LogicalAddr, VXI_SLOT, &slot);
		if(!Err_Code)
		{
			printf(" VXI-5536 found in slot %d at Logical Address %d\n",
											(int)slot, LogicalAddr);
		}
	}
	else
	{
		printf("\n *** VXI-5536 Not Found ***, Run RES_MAN");
		return 1;
	}

	//	Clear Module

	Err_Code = vxiWScmd(LogicalAddr, Clear);
	if(Err_Code)
	{
		printf("*** ERROR %d sending VXI CLEAR to Logical Address: %d\n",
											Err_Code, LogicalAddr);
		vxiErrorDisp(Err_Code);
		return 1;
	}

	//	Send Enable Passed Buffer command

	Err_Code = vxiWSqry(LogicalAddr, XFDC_ENBL_PASS_BUFR, &RtnValue);
	if(!Err_Code)
	{
		printf("LA %d, XFDC Enable Passed Buffer response: %04X\n", LogicalAddr, RtnValue);
	}
	else
	{
		printf("*** ERROR %d sending XFDC Enable Passed Buffer to Logical Address: %d\n",
											Err_Code, LogicalAddr);
		vxiErrorDisp(Err_Code);
	}

	return 0;
}


/* **************************** DataTest ****************************** */
/* ******************************************************************** */

int DataTest(void)
{
	unsigned RtnValue;
	int i, chan, intchan, tchan[4];
	long LoopCnt;
	long RxApass=0, RxBpass=0, TxApass=0, TxBpass=0;
	int Err_Code, TxChan, RxChan;
	int Tx1FDCbufr, Tx2FDCbufr, Rx1FDCbufr, Rx2FDCbufr;
	long TxRate, MemAddr;

	//	Setup module parameter per operator inputs

	Err_Code = setupChan(&TxChan, &RxChan, &TxRate);
	if(Err_Code)
		return 1;

	Rx1FDCbufr = (RxChan-1)*4 + 0;
	Rx2FDCbufr = (RxChan-1)*4 + 1;
	Tx1FDCbufr = (TxChan-1)*4 + 2;
	Tx2FDCbufr = (TxChan-1)*4 + 3;
	printf("FDC buffers: Rx A = %d, Rx B = %d, Tx A = %d, Tx B = %d\n",
				Rx1FDCbufr, Rx2FDCbufr ,Tx1FDCbufr, Tx2FDCbufr);

	tchan[0] = Tx1FDCbufr;
	tchan[1] = Tx2FDCbufr;
	tchan[2] = Rx1FDCbufr;
	tchan[3] = Rx2FDCbufr;

	XFDC_ChannelClose(LogicalAddr, Rx1FDCbufr);	// go to idle & close Rx A
	XFDC_ChannelClose(LogicalAddr, Tx1FDCbufr);	// go to idle & close Tx A

	// Init Output Buffer pair and write data

	XFDC_OpenWriteBufrPS(LogicalAddr, Tx1FDCbufr);
	A32_WriteBufr(LogicalAddr, Tx1FDCbufr, Tx1Buffer);
	A32_WriteBufr(LogicalAddr, Tx2FDCbufr, Tx2Buffer);

	//	Init Input Buffer pair

	XFDC_OpenReadBufrPS(LogicalAddr, Rx1FDCbufr);

	puts("Transmitting Data...");

	//	Check Tx buffer headers and pass 1st data buffers

	RtnValue = XFDC_GetHeader(LogicalAddr, Tx1FDCbufr);
	printf("XFDC Chan %d Header: %04X\n", Tx1FDCbufr, RtnValue);

	if((RtnValue & 0x0002))
	{
		XFDC_PassBuffer(LogicalAddr, Tx1FDCbufr);
	}

	RtnValue = XFDC_GetHeader(LogicalAddr, Tx2FDCbufr);
	printf("XFDC Chan %d Header: %04X\n", Tx2FDCbufr, RtnValue);

	if((RtnValue & 0x0002))
	{
		XFDC_PassBuffer(LogicalAddr, Tx2FDCbufr);
	}

	do		// loop until a key is hit
	{
		for(i=0; i<4; i++)			// poll all buffers
		{
			chan = tchan[i];
			RtnValue = XFDC_GetHeader(LogicalAddr, chan);
			if((RtnValue & 0x0002))		// if Tx buffer passed
			{
				if(chan == Tx1FDCbufr)
					TxApass++;
				else
					TxBpass++;

				XFDC_PassBuffer(LogicalAddr, chan);	// pass buffer back
			}
			else if((RtnValue & 0x0004))	// if Rx buffer passed
			{
				if(chan == Rx1FDCbufr)
					RxApass++;
				else
					RxBpass++;

				XFDC_PassBuffer(LogicalAddr, chan);	// pass buffer back

				printf("Passed Buffers: TA %ld, TB %ld, RA %ld, RB %ld\n",
						TxApass, TxBpass, RxApass, RxBpass);

				if(kbhit())		// check for key hit
					break;
			}
		}
	} while(!kbhit());
	getch();		// clear key hit

	if(XFDC_GetChanAddr(LogicalAddr, Rx1FDCbufr, &MemAddr))
		return Err_Code;
	else
	{
		printf("\n Rx 1 FDC Channel %d:\n", Rx1FDCbufr);
		A32bufr(MemAddr);
        pause();

		if(XFDC_GetChanAddr(LogicalAddr, Rx2FDCbufr, &MemAddr))
			return Err_Code;
		else
		{
			printf("\n Rx 2 FDC Channel %d:\n", Rx2FDCbufr);
			A32bufr(MemAddr);
        	pause();
		}
	}

	XFDC_ChannelClose(LogicalAddr, Rx1FDCbufr);	// go to idle & close Rx A
	XFDC_ChannelClose(LogicalAddr, Tx1FDCbufr);	// go to idle & close Tx A

	return Err_Code;
}


/* *************************** setupChan ****************************** */
/*      This function sets up the Rx & Tx channels per operator inputs  */
/* ******************************************************************** */

int setupChan(int *TxChan, int *RxChan, long *TxRate)
{
	unsigned RtnValue;
	int Err_Code = 0, Bits, TimOut, NumbWords, BadParam;
	char command[20], DataSize[20], ExtClk[20], InvClk[20], GatedClk[20];
	char RxStart[20], MSBsel[20], OutStr[128];
	char InitStr[] = {"*cls; *ese 48"};

	Err_Code = vxiOutput(LogicalAddr, InitStr);
	if(Err_Code)
	{
		printf("\t*** ERROR, vxiOutput - %s, Err_Code = %d\n",
														InitStr, Err_Code);
		return Err_Code;
	}
	else
		printf("\nSetup commmands sent: %s\n", InitStr);


	do
	{
		BadParam=0;
		puts("\nEnter parameters, [ENTER] for defaults, EXIT to quit");
		printf("\nEnter Transmitter channel number ([1]-4) > ");
		gets(command);				// get command from kybd
		if(!strcmpi(command,"EXIT"))
		{
			Err_Code = 1;
			break;
		}
		*TxChan = atoi(command);
		if(*TxChan == 0)
		{
			*TxChan = 1;
			printf("\tChannel set to default [%d]\n", *TxChan);
		}

		printf("\nEnter Receiver channel number (1-4 [%d]) > ", *TxChan);
		gets(command);				// get command from kybd
		if(!strcmpi(command,"EXIT"))
		{
			Err_Code = 1;
			break;
		}
		*RxChan = atoi(command);
		if(*RxChan == 0)
		{
			*RxChan = *TxChan;
			printf("\tRx Channel set to default [%d]\n", *RxChan);
		}

		printf("Select Data Bus Size (D16 | [D32]) > ");
		gets(DataSize);				// get command from kybd
		if(strlen(DataSize) == 0)
		{
			strcpy(DataSize, "D32");
			printf("\tData Bus Size set to default [%s]\n", DataSize);
		}

		printf("Select Words Per Message (1, [2], 4 > ");
		gets(command);				// get command from kybd
		NumbWords = atoi(command);
		if(NumbWords == 0)
		{
			NumbWords = 2;
			printf("\tWords Per Message set to default [%d]\n", NumbWords);
		}

		printf("Select External Clock (ON | [OFF]) > ");
		gets(ExtClk);				// get command from kybd
		if(strlen(ExtClk) == 0)
		{
			strcpy(ExtClk, "OFF");
			printf("\tExternal Clock set to default [%s]\n", ExtClk);
		}

		printf("Select Clock Inversion (ON | [OFF]) > ");
		gets(InvClk);				// get command from kybd
		if(strlen(InvClk) == 0)
		{
			strcpy(InvClk, "OFF");
			printf("\tClock Invert set to default [%s]\n", InvClk);
		}

		printf("Select Tx Gated Clock ([ON] | OFF) > ");
		gets(GatedClk);				// get command from kybd
		if(strlen(GatedClk) == 0)
		{
			strcpy(GatedClk, "ON");
			printf("\tTx Gated Clock mode set to default [%s]\n", GatedClk);
		}

		printf("Select Rx Start Mode ([GATE] | CLOCK) > ");
		gets(RxStart);				// get command from kybd
		if(strlen(RxStart) == 0)
		{
			strcpy(RxStart, "GATE");
			printf("\tRx Start mode set to default [%s]\n", RxStart);
		}

		printf("Select MSB first ([ON] | OFF) > ");
		gets(MSBsel);				// get command from kybd
		if(strlen(MSBsel) == 0)
		{
			strcpy(MSBsel, "ON");
			printf("\tMSB first set to default [%s]\n", MSBsel);
		}

		printf("Enter Number of bits (1 - [64]) > ");
		gets(command);				// get command from kybd
		Bits = atoi(command);
		if(Bits == 0)
		{
			Bits = 64;
			printf("\tBits set to default [%d]\n", Bits);
		}

		printf("Enter Time Out (0 - 500) [200] > ");
		gets(command);				// get command from kybd
		TimOut = atoi(command);
		if(TimOut == 0)
		{
			TimOut = 200;
			printf("\tTime Out set to default [%d]\n", TimOut);
		}

		printf("Enter Transmitter rate (153 - [1000000]) > ");
		gets(command);				// get command from kybd
		*TxRate = atol(command);
		if(*TxRate == 0)
		{
			*TxRate = 1000000;
			printf("\tRate set to default [%ld]\n", *TxRate);
		}

		sprintf(OutStr,"*cls; c %d; bdw %s; bws %d; rck %s; str %s; bits %d; rsz 80",
							*RxChan, DataSize, NumbWords, InvClk, RxStart, Bits);

		Err_Code = vxiOutput(LogicalAddr, OutStr);
		if(Err_Code)
		{
			printf("\t*** ERROR, vxiOutput - %s, Err_Code = %d\n",
														OutStr, Err_Code);
			return Err_Code;
		}
		else
			printf("Rx cmds: %s\n", OutStr);

		msDelay(100);

 		vxiWSqry(LogicalAddr, ReadSTB, &RtnValue);
		if((RtnValue & 32))
		{
			puts("\t*** ERROR - Bad parameter ***");
			BadParam = 1;
		}

		sprintf(OutStr,"c %d; bdw %s; bws %d; trt %ld; bits %d; ext %s; tgc %s; msb %s; tmo %d",
							*TxChan, DataSize, NumbWords, *TxRate, Bits, ExtClk, GatedClk, MSBsel, TimOut);

		Err_Code = vxiOutput(LogicalAddr, OutStr);
		if(Err_Code)
		{
			printf("\t*** ERROR, vxiOutput - %s, Err_Code = %d\n",
														OutStr, Err_Code);
			return Err_Code;
		}
		else
			printf("Tx cmds: %s\n", OutStr);

		msDelay(100);

 		vxiWSqry(LogicalAddr, ReadSTB, &RtnValue);
		if((RtnValue & 32))
		{
			puts("\t*** ERROR - Bad parameter ***");
			BadParam = 1;
		}

	} while(BadParam);

	return Err_Code;
}


/* *************************** A32_WriteBufr ************************** */
/*      This function writes a FDC buffer in A32 memory space.			*/
/* ******************************************************************** */

void A32_WriteBufr(int LogicalAddr, int chan, int *buffer)
{
	int Err_Code, i, size;
	unsigned long MemAddr;

	size = *buffer;

	if(XFDC_GetChanAddr(LogicalAddr, chan, &MemAddr))
		exit(Err_Code);

	for(i=0; i<size; i+=2)
	{
		if(vxiWrA32Reg(MemAddr, i+4, *buffer++))
			break;
	}
}


/* ************************* XFDC_PassBuffer ************************** */
/*      This function passes a FDC channel buffer.						*/
/* ******************************************************************** */

void XFDC_PassBuffer(int LogicalAddr, int chan)
{
	int Err_Code;

	Err_Code = vxiWScmd(LogicalAddr, XFDC_PASSED_BUFR+chan);

	if(Err_Code)
	{
		printf("*** ERROR %d sending X Passed Buffer:%d to Logical Address: %d\n",
											Err_Code, chan, LogicalAddr);
		vxiErrorDisp(Err_Code);
	}
}


/* *********************** XFDC_ChannelClose ************************** */
/*      This function closes a XFDC channel.							*/
/* ******************************************************************** */

void XFDC_ChannelClose(int LogicalAddr, int chan)
{
	unsigned RtnValue;
	int Err_Code;

	Err_Code = vxiWSqry(LogicalAddr, XFDC_GO_TO_IDLE_IM+chan, &RtnValue);
	if(!Err_Code)
		printf("LA %d, Go to Idle IM:%d response: %04X\n",
										LogicalAddr, chan, RtnValue);
	else
	{
		printf("*** ERROR %d sending X Go to Idle IM:%d to Logical Address: %d\n",
											Err_Code, chan, LogicalAddr);
		vxiErrorDisp(Err_Code);
	}

	Err_Code = vxiWSqry(LogicalAddr, XFDC_CHAN_CLS+chan, &RtnValue);
	if(!Err_Code)
		printf("LA %d, X FDC Chan Close:%d response: %04X\n",
											LogicalAddr, chan, RtnValue);
	else
	{
		printf("*** ERROR %d sending X Chan Close:%d to Logical Address: %d\n",
											Err_Code, chan, LogicalAddr);
		vxiErrorDisp(Err_Code);
	}
}


/* ********************** XFDC_OpenWriteBufrPS ************************ */
/*      This function opens a FDC write buffer in Pair & Stream mode.	*/
/* ******************************************************************** */

void XFDC_OpenWriteBufrPS(int LogicalAddr, int chan)
{
	unsigned RtnValue;
	int Err_Code;

	Err_Code = vxiWSqry(LogicalAddr, XFDC_CHAN_INIT+chan, &RtnValue);
	if(!Err_Code)
		printf("LA %d, X FDC Chan Init:%d response: %04X\n",
											LogicalAddr, chan, RtnValue);
	else
	{
		printf("*** ERROR %d sending X Chan Init:%d to Logical Address: %d\n",
											Err_Code, chan, LogicalAddr);
		vxiErrorDisp(Err_Code);
	}

	Err_Code = vxiWSqry(LogicalAddr, XFDC_XFER_TO_SRV_PS+chan, &RtnValue);
	if(!Err_Code)
		printf("LA %d, X FDC Xfer to Srv PS:%d response: %04X\n",
											LogicalAddr, chan, RtnValue);
	else
	{
		printf("*** ERROR %d sending X Xfer to Srv PS:%d to Logical Address: %d\n",
											Err_Code, chan, LogicalAddr);
		vxiErrorDisp(Err_Code);
	}
}


/* **************************** XFDC_GetHeader ************************ */
/*      This function reads the X FDC buffer header bits. 				*/
/* ******************************************************************** */

int XFDC_GetHeader(int LogicalAddr, int chan)
{
	long ChanAddr;
	unsigned RtnValue;
	int Err_Code;

	Err_Code = vxiWSqry(LogicalAddr, XFDC_CHAN_ADDR_HIGH+chan, &RtnValue);
	if(!Err_Code)
	{
		ChanAddr = (long)RtnValue << 16;
	}
	else
	{
		printf("*** ERROR %d sending X FDC Chan Addr High:%d to Logical Address: %d\n",
 											Err_Code, chan, LogicalAddr);
		vxiErrorDisp(Err_Code);
		return -1;
	}

	Err_Code = vxiWSqry(LogicalAddr, XFDC_CHAN_ADDR_LOW+chan, &RtnValue);
	if(!Err_Code)
	{
		ChanAddr |= RtnValue;
	}
	else
	{
		printf("*** ERROR %d sending X FDC Chan Addr Low:%d to Logical Address: %d\n",
											Err_Code, chan, LogicalAddr);
		vxiErrorDisp(Err_Code);
		return -1;
	}

	Err_Code = vxiRdA32Reg(ChanAddr, 0, &RtnValue);
	if(Err_Code)
	{
		printf("*** ERROR %d reading from Address: %08lX\n",
												Err_Code, ChanAddr+2);
		vxiErrorDisp(Err_Code);
		return -1;
	}

	return RtnValue;
}


/* *********************** XFDC_OpenReadBufrPS ************************ */
/*      This function opens a FDC read buffer in Pair & Stream mode.	*/
/* ******************************************************************** */

void XFDC_OpenReadBufrPS(int LogicalAddr, int chan)
{
	unsigned RtnValue;
	int Err_Code;

	Err_Code = vxiWSqry(LogicalAddr, XFDC_CHAN_INIT+chan, &RtnValue);
	if(!Err_Code)
		printf("LA %d, X FDC Chan Init:%d response: %04X\n",
											LogicalAddr, chan, RtnValue);
	else
	{
		printf("*** ERROR %d sending X Chan Init:%d to Logical Address: %d\n",
											Err_Code, chan, LogicalAddr);
		vxiErrorDisp(Err_Code);
	}

	Err_Code = vxiWSqry(LogicalAddr, XFDC_XFER_TO_CMD_PS+chan, &RtnValue);
	if(!Err_Code)
		printf("LA %d, X FDC Xfer to Cmdr PS:%d response: %04X\n",
											LogicalAddr, chan, RtnValue);
	else
	{
		printf("*** ERROR %d sending X Xfer to Cmdr PS:%d to Logical Address: %d\n",
											Err_Code, chan, LogicalAddr);
		vxiErrorDisp(Err_Code);
	}
}


/* ************************* XFDC_GetChanAddr ************************* */
/*      This function get a XFDC buffer address.     					*/
/* ******************************************************************** */

int XFDC_GetChanAddr(int LogicalAddr, int chan, unsigned long *MemAddr)
{
	unsigned RtnValue;
	int Err_Code;

	Err_Code = vxiWSqry(LogicalAddr, XFDC_CHAN_ADDR_HIGH+chan, &RtnValue);
	if(!Err_Code)
	{
		*MemAddr = (long)RtnValue << 16;
	}
	else
	{
		printf("*** ERROR %d sending XFDC Chan Addr High:%d to Logical Address: %d\n",
											Err_Code, chan, LogicalAddr);
		vxiErrorDisp(Err_Code);
	}

	Err_Code = vxiWSqry(LogicalAddr, XFDC_CHAN_ADDR_LOW+chan, &RtnValue);
	if(!Err_Code)
	{
		*MemAddr |= RtnValue;
	}
	else
	{
		printf("*** ERROR %d sending XFDC Chan Addr Low:%d to Logical Address: %d\n",
											Err_Code, chan, LogicalAddr);
		vxiErrorDisp(Err_Code);
	}
	return Err_Code;
}


/* **************************** dispXFDCbuf *************************** */
/*      This function reads an A32 buffer and displays it.    			*/
/*																		*/
/*	Inputs:		A32 memory address (long)								*/
/*	Returns:	VXI library error code (int)							*/
/* ******************************************************************** */

int A32bufr(long MemAddr)
{
	long i;
	unsigned RdData;
	int Err_Code;

	MemAddr -= (MemAddr % 16);		// justify row address

	printf("\tADDRESS  ---------------- DATA -----------------");
	for(i=0; i<256; i+=2)
	{
		if(i % 16 == 0 || i == MemAddr)	// if time for new row
			printf("\n\t%08lX  ", i+MemAddr);

		Err_Code = vxiRdA32Reg(MemAddr, i, &RdData);  	// read from memory
		if(Err_Code)  // if VXI Error
		{
			vxiErrorDisp(Err_Code);
			break;
		}
		else
			printf("%04X ", RdData);
	}
	puts("");
}


/* **************************** pause ********************************* */
/*      This function displays a message & waits for a key press.		*/
/* ******************************************************************** */

void pause(void)
{
	puts("PAUSED: press any key to continue...");
	while(!kbhit());
	getch();
}
