/* ******************************************************************** */
/*                       VXI WORD SERIAL COMMANDS                       */
/*                                                                      */
/*   Filename:  VXICMDS.H                                               */
/*                                                                      */
/*   This file contains the VXI Word Serial Command code equates        */
/*   for the VXI interface.  If the command code ends with '00' then    */
/*   the parameter field must be filled in before sending the command.  */
/*   (Q) indicates the command is a query or has a reponse which must   */
/*   be read (send with vxiWSqry() library function).                   */
/*   See the VXIbus Specification for more information on Word Serial   */
/*   Commands  (Section E.1).                                           */
/*                                                                      */
/*   Revised:   05/06/94    CB                                          */
/*				12/07/94	CB	- added new error codes					*/
/* ******************************************************************** */

#define AbortNormOp     0xc8ff            /* Abort Normal Operation (Q) */
#define AssgnHandLine   0xa900            /* Assign Handler Line (Q) */
#define AssgnIntrLine   0xaa00            /* Assign Interrupter Line (Q) */
#define AysncModeCntl   0xa800            /* Asynchronous Mode Control (Q) */
#define BeginNormOp     0xfcff            /* Begin Normal Operation (Q) */
#define BeginNormOpT    0xfdff            /* Begin Normal Operation Top (Q) */
#define ByteAvail       0xbc00            /* Byte Available */
#define ByteAvailEnd    0xbd00            /* Byte Available End of Mesg */
#define ByteReq         0xdeff            /* Byte Request (Q) */
#define Clear           0xffff            /* Clear */
#define ClearLock       0xefff            /* Clear Lock */
#define ControlEvent    0xaf00            /* Control Event (Q) */
#define ControlResponse 0x8f00            /* Control Response (Q) */
#define EndNormOp       0xc9ff            /* End Normal Operation (Q) */
#define GrantDev        0xbf00            /* Grant Device */
#define IdentCmdr       0xbe00            /* Identify Commander */
#define ReadHandlers    0xc7ff            /* Read Handlers (Q) */
#define ReadHandLine    0x8c00            /* Read Handler Line (Q) */
#define ReadIntrLine    0x8d00            /* Read Interrupter Line (Q) */
#define ReadIntrpters   0xcaff            /* Read Interrupters (Q) */
#define ReadMODID       0xccff            /* Read MODID (Q) */
#define ReadProto       0xdfff            /* Read Protocol (Q) */
#define ReadProtoErr    0xcdff            /* Read Protocol Error (Q) */
#define ReadSTB         0xcfff            /* Read STB (Q) */
#define ReadServArea    0xceff            /* Read Servant Area (Q) */
#define ReleaseDev      0x8e00            /* Release Device (Q) */
#define SetLock         0xeeff            /* Set Lock */
#define SetLowerMODID   0xae00            /* Set Lower MODID (Q) */
#define SetUpperMODID   0xad00            /* Set Upper MODID (Q) */
#define Trigger         0xedff            /* Trigger */

//      ICS VXI-5535 Special Word Serial Commands

#define ControlMode     0x0100            /* Select Control Mode */
#define DataMode        0x0000            /* Select Data Mode */
#define ArmTrigger      0x0200            /* Arm Trigger, buf/TTL line */


//      Word Serial Command Response Codes

#define WScmdOK         0xfffe          /* Word Serial command OK */
#define WScmdFAIL       0x7ffe          /* Word Serial command FAILED */


//      Read Protocol Error Response Codes

#define PE_None			0xffff          // no error
#define PE_MultQry		0xfffd          // multiple queries
#define PE_UnSpCmd		0xfffc          // unsupported cmd
#define PE_DIR			0xfffb          // DIR violation
#define PE_DOR			0xfffa          // DOR violation
#define PE_RdRdy		0xfff9          // Read Ready violation
#define PE_WtRdy		0xfff8          // Write Ready violation


//      VXI Register offsets

#define ID_Register  0x00              // ID Register
#define Device_Type  0x02              // Device Type
#define Status_Cntl  0x04              // Status / Control Register
#define Offset_Reg   0x06              // (Offset Register)
#define Protocol_Reg 0x08              // Protocol (/Signal) Register
#define Response_Reg 0x0A              // Response (/Data Extended)
#define Data_High    0x0C              // (Data High Register)
#define Data_Low     0x0E              // Data Low Register
#define A24_Ptr_H    0x10              // (A24 Pointer High)
#define A24_Ptr_L    0x12              // (A24 Pointer Low)
#define A32_Ptr_H    0x14              // (A32 Pointer High)
#define A32_Ptr_L    0x16              // (A32 Pointer Low)

//      VXI Library Error Codes

#define TermCharRcv -2					// termination char rcv'd
#define MaxNumChar  -1					// maximum number of chars rcv'd
#define No_Error    0                   // no error found
#define Buss_Err    1                   // bus error (BERR)
#define WrtRdy_Err  2                   // write ready time out error
#define RdRdy_Err   3                   // read ready time out error
#define DIR_Err     4                   // DIR time out error
#define DOR_Err     5                   // DOR time out error
#define Proto_Err   6                   // protocol error (*ERR)
#define BusGrnt_Err 7                   // bus grant error
#define RspRead_Err 8                   // Response not read error
#define LA_Err      9                   // Logical Address error
#define Reg_Err     10                  // Register offset error
#define Addr_Err	11					// A24/A32 address error
#define Count_Err	12					// byte count error
#define Param_Err	13					// parameter error


//	VXI Attributes defines (same codes as VTL)

#define	VXI_TMO_VALUE			0       // timeout value in mSec
#define	VXI_SEND_END_EN			1       // send end enable
#define	VXI_SUPPRESS_END_EN		2       // suppress end on read enable
#define	VXI_TERMCHAR			3       // termination char
#define	VXI_TERMCHAR_EN			4       // read termination on char enable
#define	VXI_MANF_ID       		9       // manufacturers id
#define	VXI_MODEL_CODE			10      // model code
#define	VXI_SLOT				11      // slot number
#define	VXI_MAP_BASE			12      // map base
#define	VXI_MAP_SIZE			13      // map size

